@@ -47,6 +47,41 @@ import gnomemusic.utils as utils
 logger = logging.getLogger(__name__)
 
 
+class LookupQueue(object):
+    """A queue for IO operations"""
+
+    _max_simultaneous_lookups = 12
+    _lookup_queue = []
+    _n_lookups = 0
+
+    @classmethod
+    @log
+    def push(cls, cache, item, art_size, callback, itr):
+        """Push a lookup counter or queue the lookup if needed"""
+
+        # If reached the limit, queue the operation.
+        if cls._n_lookups >= cls._max_simultaneous_lookups:
+            cls._lookup_queue.append((cache, item, art_size, callback, itr))
+            return False
+        else:
+            cls._n_lookups += 1
+            return True
+
+    @classmethod
+    @log
+    def pop(cls):
+        """Pops a lookup counter and consume the lookup queue if needed"""
+
+        cls._n_lookups -= 1
+
+        # An available lookup slot appeared! Let's continue looking up
+        # artwork then.
+        if (cls._n_lookups < cls._max_simultaneous_lookups
+                and cls._lookup_queue):
+            (cache, item, art_size, callback, itr) = cls._lookup_queue.pop(0)
+            cache.lookup(item, art_size, callback, itr)
+
+
 @log
 def _make_icon_frame(pixbuf, art_size=None, scale=1):
     border = 3 * scale
@@ -235,10 +270,11 @@ class AlbumArtCache(GObject.GObject):
         :param callback: Callback function when retrieved
         :param itr: Iter to return with callback
         """
-        self._lookup_local(item, callback, itr, art_size)
+        if LookupQueue.push(self, item, art_size, callback, itr):
+            self._lookup_local(item, art_size, callback, itr)
 
     @log
-    def _lookup_local(self, item, callback, itr, art_size):
+    def _lookup_local(self, item, art_size, callback, itr):
         """Checks if there is already a local art file, if not calls
         the embedded lookup function"""
         album = utils.get_album_title(item)
@@ -269,6 +305,10 @@ class AlbumArtCache(GObject.GObject):
             return
 
         def do_callback(pixbuf):
+
+            # Lookup finished, decrease the counter
+            LookupQueue.pop()
+
             if not pixbuf:
                 surface = DefaultIcon(self._scale).get(DefaultIcon.Type.music,
                                                        art_size)
@@ -298,11 +338,17 @@ class AlbumArtCache(GObject.GObject):
             do_callback(None)
             return
 
-        self._lookup_embedded(item, callback, itr, art_size)
+        # When we reach here because it fails to retrieve the artwork,
+        # do a long round trip (either through _lookup_embedded or
+        # _lookup_remote) and call self.lookup() again. Thus, decrease
+        # global lookup counter.
+        LookupQueue.pop()
+
+        self._lookup_embedded(item, art_size, callback, itr)
 
     @log
     def _discovered_cb(self, discoverer, info, error):
-        item, callback, itr, art_size, cache_path = \
+        item, art_size, callback, itr, cache_path = \
             self._discoverer_items[info.get_uri()]
 
         album = utils.get_album_title(item)
@@ -320,7 +366,12 @@ class AlbumArtCache(GObject.GObject):
 
             self.lookup(item, art_size, callback, itr)
 
-        if error is not None:
+        # FIXME: tags should not return as None, but it sometimes is.
+        # So as a workaround until we figure out what is wrong check
+        # for it.
+        # https://bugzilla.gnome.org/show_bug.cgi?id=780980
+        if (error is not None
+                or tags is None):
             art_retrieved(False)
             return
 
@@ -361,10 +412,10 @@ class AlbumArtCache(GObject.GObject):
             logger.warn("Trying to process misc albumart: %s, %s",
                         err.__class__, err)
 
-        self._lookup_remote(item, callback, itr, art_size)
+        self._lookup_remote(item, art_size, callback, itr)
 
     @log
-    def _lookup_embedded(self, item, callback, itr, art_size):
+    def _lookup_embedded(self, item, art_size, callback, itr):
         """Lookup embedded cover
 
         Lookup embedded art through Gst.Discoverer. If found
@@ -378,12 +429,12 @@ class AlbumArtCache(GObject.GObject):
         if not success:
             self._lookup_remote(item, callback, itr, art_size)
 
-        self._discoverer_items[item.get_url()] = [item, callback, itr,
-                                                  art_size, cache_path]
+        self._discoverer_items[item.get_url()] = [item, art_size, callback,
+                                                  itr, cache_path]
         self._discoverer.discover_uri_async(item.get_url())
 
     @log
-    def _lookup_remote(self, item, callback, itr, art_size):
+    def _lookup_remote(self, item, art_size, callback, itr):
         """Lookup remote art
 
         Lookup remote art through Grilo and if found copy locally. Call
