@@ -265,7 +265,7 @@ class Player(GObject.GObject):
         install_ctx = GstPbutils.InstallPluginsContext.new()
 
         if self._gst_plugins_base_check_version(1, 5, 0):
-            install_ctx.set_desktop_id('gnome-music.desktop')
+            install_ctx.set_desktop_id('org.gnome.Music.desktop')
             install_ctx.set_confirm_search(confirm_search)
 
             startup_id = '_TIME%u' % Gtk.get_current_event_time()
@@ -828,6 +828,7 @@ class Player(GObject.GObject):
             self._on_progress_scale_draw)
         self._seek_timeout = None
         self._old_progress_scale_value = 0.0
+        self.progressScale.set_increments(300, 600)
 
     def _on_progress_scale_seek_finish(self, value):
         """Prevent stutters when seeking with infinitesimal amounts"""
@@ -835,6 +836,7 @@ class Player(GObject.GObject):
         round_digits = self.progressScale.get_property('round-digits')
         if self._old_progress_scale_value != round(value, round_digits):
             self.on_progress_scale_change_value(self.progressScale)
+            self._old_progress_scale_value = round(value, round_digits)
         return False
 
     def _on_progress_scale_seek(self, scale, scroll_type, value):
@@ -847,9 +849,14 @@ class Player(GObject.GObject):
             GLib.source_remove(self._seek_timeout)
 
         Gtk.Range.do_change_value(scale, scroll_type, value)
-        self._seek_timeout = GLib.timeout_add(100,
-                                              self._on_progress_scale_seek_finish,
-                                              value)
+        if scroll_type == Gtk.ScrollType.JUMP:
+            self._seek_timeout = GLib.timeout_add(
+                100, self._on_progress_scale_seek_finish, value)
+        else:
+            # scroll with keys, hence no smoothing
+            self._on_progress_scale_seek_finish(value)
+            self._update_position_callback()
+
         return True
 
     @log
