@@ -24,7 +24,8 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-
+import gi
+gi.require_version('Gd', '1.0')
 from gi.repository import Gtk, Gd, GObject, Pango, GLib
 from gettext import gettext as _
 from gnomemusic.grilo import grilo
